typedef struct
{
	float x;
	float y;
	float z;
}Vector3;

const Vector3 Zero={0, 0, 0};
const Vector3 One={1, 1, 1};
const Vector3 Up={0, 1, 0};
const Vector3 Right={1, 0, 0};
const Vector3 Down={0, -1, 0};
const Vector3 Left={-1, 0, 0};

void VectorAssign(Vector3* v, float x, float y, float z)
{
	v->x = x;
	v->y = y;
	v->z = z;
}

void VectorEquals(Vector3* lvalue, Vector3* rvalue)
{
	(*lvalue)=(*rvalue);
}

float Magnitude(Vector3* v)
{
	return (sqrt(pow(v->x, 2) + pow(v->y, 2) + pow(v->z, 2)));
}

Vector3 Add(Vector3* v1, Vector3* v2)
{
	Vector3 result;
	VectorAssign(&result,v1->x + v2->x, v1->y + v2->y, v1->z + v2->z);
	return result;
}

Vector3 Subtract(Vector3* v1, Vector3* v2)
{
    Vector3 result;
	VectorAssign(&result,v1->x - v2->x, v1->y - v2->y, v1->z - v2->z);
	return result;
}

Vector3 MultiplyScalar(Vector3* v, float val)
{
	Vector3 result;
	VectorAssign(&result,v->x * val, v->y * val, v->z * val);
	return result;
}

void Normalize(Vector3* v)
{
	float mag = Magnitude(v);
	v->x =v->x/ mag;
	v->y =v->y/ mag;
	v->z =v->z/ mag;
}

float Dot(Vector3* v1, Vector3* v2)
{
	return ((v1->x * v2->x) + (v1->y * v2->y) + (v1->z * v2->z));
}

Vector3 Cross(Vector3* v1, Vector3* v2)
{
	Vector3 result;
	VectorAssign(&result, ((v1->y * v2->z) - (v1->z * v2->y)), -((v1->x * v2->z) - (v1->z * v2->x)), ((v1->x * v2->y) - (v1->y * v2->x)));
	return result;
}

float Angle(Vector3* v1, Vector3* v2)
{
	float mag1 = Magnitude(v1);
	float mag2 = Magnitude(v2);
	float dot = Dot(v1, v2);
	float result = dot / (mag1 * mag2);
    return (acos(result) * 180 / 3.141592654f);
}